///<reference path="../.d.ts"/>
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var path = require("path");
var shell = require("shelljs");
var Future = require("fibers/future");
var constants = require("../constants");
var semver = require("semver");
var projectServiceBaseLib = require("./platform-project-service-base");
var androidDebugBridgePath = require("../common/mobile/android/android-debug-bridge");
var AndroidProjectService = (function (_super) {
    __extends(AndroidProjectService, _super);
    function AndroidProjectService($androidEmulatorServices, $androidToolsInfo, $childProcess, $errors, $fs, $hostInfo, $logger, $options, $projectData, $projectDataService, $sysInfo, $mobileHelper, $injector) {
        _super.call(this, $fs, $projectData, $projectDataService);
        this.$androidEmulatorServices = $androidEmulatorServices;
        this.$androidToolsInfo = $androidToolsInfo;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$hostInfo = $hostInfo;
        this.$logger = $logger;
        this.$options = $options;
        this.$sysInfo = $sysInfo;
        this.$mobileHelper = $mobileHelper;
        this.$injector = $injector;
        this._platformData = null;
        this._androidProjectPropertiesManagers = Object.create(null);
    }
    Object.defineProperty(AndroidProjectService.prototype, "platformData", {
        get: function () {
            if (!this._platformData) {
                var projectRoot = path.join(this.$projectData.platformsDir, "android");
                this._platformData = {
                    frameworkPackageName: "tns-android",
                    normalizedPlatformName: "Android",
                    appDestinationDirectoryPath: path.join(projectRoot, "src", "main", "assets"),
                    platformProjectService: this,
                    emulatorServices: this.$androidEmulatorServices,
                    projectRoot: projectRoot,
                    deviceBuildOutputPath: path.join(projectRoot, "build", "outputs", "apk"),
                    validPackageNamesForDevice: [
                        (this.$projectData.projectName + "-debug.apk"),
                        (this.$projectData.projectName + "-release.apk")
                    ],
                    frameworkFilesExtensions: [".jar", ".dat", ".so"],
                    configurationFileName: "AndroidManifest.xml",
                    configurationFilePath: path.join(projectRoot, "src", "main", "AndroidManifest.xml"),
                    mergeXmlConfig: [{ "nodename": "manifest", "attrname": "*" }, { "nodename": "application", "attrname": "*" }]
                };
            }
            return this._platformData;
        },
        enumerable: true,
        configurable: true
    });
    AndroidProjectService.prototype.getAppResourcesDestinationDirectoryPath = function (frameworkVersion) {
        var _this = this;
        return (function () {
            if (_this.canUseGradle(frameworkVersion).wait()) {
                return path.join(_this.platformData.projectRoot, "src", "main", "res");
            }
            return path.join(_this.platformData.projectRoot, "res");
        }).future()();
    };
    AndroidProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            _this.validatePackageName(_this.$projectData.projectId);
            _this.validateProjectName(_this.$projectData.projectName);
            _this.checkJava().wait();
            _this.checkAndroid().wait();
        }).future()();
    };
    AndroidProjectService.prototype.createProject = function (frameworkDir, frameworkVersion) {
        var _this = this;
        return (function () {
            if (semver.lt(frameworkVersion, AndroidProjectService.MIN_RUNTIME_VERSION_WITH_GRADLE)) {
                _this.$errors.failWithoutHelp("The NativeScript CLI requires Android runtime " + AndroidProjectService.MIN_RUNTIME_VERSION_WITH_GRADLE + " or later to work properly.");
            }
            _this.checkGradle().wait();
            _this.$fs.ensureDirectoryExists(_this.platformData.projectRoot).wait();
            var androidToolsInfo = _this.$androidToolsInfo.getToolsInfo().wait();
            var targetSdkVersion = androidToolsInfo.targetSdkVersion;
            _this.$logger.trace("Using Android SDK '" + targetSdkVersion + "'.");
            if (_this.$options.symlink) {
                _this.symlinkDirectory("build-tools", _this.platformData.projectRoot, frameworkDir).wait();
                _this.symlinkDirectory("libs", _this.platformData.projectRoot, frameworkDir).wait();
                _this.symlinkDirectory("src", _this.platformData.projectRoot, frameworkDir).wait();
                _this.$fs.symlink(path.join(frameworkDir, "build.gradle"), path.join(_this.platformData.projectRoot, "build.gradle")).wait();
                _this.$fs.symlink(path.join(frameworkDir, "settings.gradle"), path.join(_this.platformData.projectRoot, "settings.gradle")).wait();
            }
            else {
                _this.copy(_this.platformData.projectRoot, frameworkDir, "build-tools libs src", "-R");
                _this.copy(_this.platformData.projectRoot, frameworkDir, "build.gradle settings.gradle", "-f");
            }
            _this.cleanResValues(targetSdkVersion, frameworkVersion).wait();
        }).future()();
    };
    AndroidProjectService.prototype.cleanResValues = function (targetSdkVersion, frameworkVersion) {
        var _this = this;
        return (function () {
            var resDestinationDir = _this.getAppResourcesDestinationDirectoryPath(frameworkVersion).wait();
            var directoriesInResFolder = _this.$fs.readDirectory(resDestinationDir).wait();
            var directoriesToClean = directoriesInResFolder
                .map(function (dir) {
                return {
                    dirName: dir,
                    sdkNum: parseInt(dir.substr(AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX.length))
                };
            })
                .filter(function (dir) { return dir.dirName.match(AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX)
                && dir.sdkNum
                && (!targetSdkVersion || (targetSdkVersion < dir.sdkNum)); })
                .map(function (dir) { return path.join(resDestinationDir, dir.dirName); });
            _this.$logger.trace("Directories to clean:");
            _this.$logger.trace(directoriesToClean);
            Future.wait(_.map(directoriesToClean, function (dir) { return _this.$fs.deleteDirectory(dir); }));
        }).future()();
    };
    AndroidProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            var manifestPath = _this.platformData.configurationFilePath;
            shell.sed('-i', /__PACKAGE__/, _this.$projectData.projectId, manifestPath);
            var stringsFilePath = path.join(_this.getAppResourcesDestinationDirectoryPath().wait(), 'values', 'strings.xml');
            shell.sed('-i', /__NAME__/, _this.$projectData.projectName, stringsFilePath);
            shell.sed('-i', /__TITLE_ACTIVITY__/, _this.$projectData.projectName, stringsFilePath);
            var gradleSettingsFilePath = path.join(_this.platformData.projectRoot, "settings.gradle");
            shell.sed('-i', /__PROJECT_NAME__/, _this.$projectData.projectId.split(".")[2], gradleSettingsFilePath);
            shell.sed('-i', /__APILEVEL__/, _this.$options.sdk || _this.$androidToolsInfo.getToolsInfo().wait().compileSdkVersion.toString(), manifestPath);
        }).future()();
    };
    AndroidProjectService.prototype.afterCreateProject = function (projectRoot) {
        return Future.fromResult();
    };
    AndroidProjectService.prototype.canUpdatePlatform = function (currentVersion, newVersion) {
        return Future.fromResult(true);
    };
    AndroidProjectService.prototype.updatePlatform = function (currentVersion, newVersion, canUpdate, addPlatform, removePlatforms) {
        var _this = this;
        return (function () {
            if (semver.eq(newVersion, AndroidProjectService.MIN_RUNTIME_VERSION_WITH_GRADLE)) {
                var platformLowercase = _this.platformData.normalizedPlatformName.toLowerCase();
                removePlatforms([platformLowercase.split("@")[0]]).wait();
                addPlatform(platformLowercase).wait();
                return false;
            }
            return true;
        }).future()();
    };
    AndroidProjectService.prototype.buildProject = function (projectRoot, buildConfig) {
        var _this = this;
        return (function () {
            if (_this.canUseGradle().wait()) {
                _this.checkGradle().wait();
                var androidToolsInfo = _this.$androidToolsInfo.getToolsInfo().wait();
                var compileSdk = androidToolsInfo.compileSdkVersion;
                var targetSdk = _this.getTargetFromAndroidManifest().wait() || compileSdk;
                var buildToolsVersion = androidToolsInfo.buildToolsVersion;
                var appCompatVersion = androidToolsInfo.supportRepositoryVersion;
                var buildOptions = ["buildapk",
                    ("-PcompileSdk=android-" + compileSdk),
                    ("-PtargetSdk=" + targetSdk),
                    ("-PbuildToolsVersion=" + buildToolsVersion),
                    ("-PsupportVersion=" + appCompatVersion),
                ];
                if (_this.$options.release) {
                    buildOptions.push("-Prelease");
                    buildOptions.push("-PksPath=" + _this.$options.keyStorePath);
                    buildOptions.push("-Palias=" + _this.$options.keyStoreAlias);
                    buildOptions.push("-Ppassword=" + _this.$options.keyStoreAliasPassword);
                    buildOptions.push("-PksPassword=" + _this.$options.keyStorePassword);
                }
                if (buildConfig && buildConfig.runSbGenerator) {
                    buildOptions.push("-PrunSBGenerator");
                }
                _this.spawn("gradle", buildOptions, { stdio: "inherit", cwd: _this.platformData.projectRoot }).wait();
            }
            else {
                _this.checkAnt().wait();
                var args = _this.getAntArgs(_this.$options.release ? "release" : "debug", projectRoot);
                _this.spawn('ant', args).wait();
                _this.platformData.deviceBuildOutputPath = path.join(_this.platformData.projectRoot, "bin");
            }
        }).future()();
    };
    AndroidProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(this.platformData.appDestinationDirectoryPath, constants.APP_FOLDER_NAME));
    };
    AndroidProjectService.prototype.addLibrary = function (libraryPath) {
        var _this = this;
        return (function () {
            if (_this.$fs.exists(path.join(libraryPath, "project.properties")).wait()) {
                _this.$errors.failWithoutHelp("Unable to add android library. You can use `library add` command only with path to folder containing one or more .jar files.");
            }
            var name = path.basename(libraryPath);
            var targetLibPath = _this.getLibraryPath(name);
            var targetPath = path.dirname(targetLibPath);
            _this.$fs.ensureDirectoryExists(targetPath).wait();
            shell.cp("-f", path.join(libraryPath, "*.jar"), targetPath);
            var projectLibsDir = path.join(_this.platformData.projectRoot, "libs");
            _this.$fs.ensureDirectoryExists(projectLibsDir).wait();
            shell.cp("-f", path.join(libraryPath, "*.jar"), projectLibsDir);
        }).future()();
    };
    AndroidProjectService.prototype.getFrameworkFilesExtensions = function () {
        return [".jar", ".dat"];
    };
    AndroidProjectService.prototype.prepareProject = function () {
        return Future.fromResult();
    };
    AndroidProjectService.prototype.prepareAppResources = function (appResourcesDirectoryPath) {
        var _this = this;
        return (function () {
            var resourcesDirPath = path.join(appResourcesDirectoryPath, _this.platformData.normalizedPlatformName);
            var valuesDirRegExp = /^values/;
            var resourcesDirs = _this.$fs.readDirectory(resourcesDirPath).wait().filter(function (resDir) { return !resDir.match(valuesDirRegExp); });
            _.each(resourcesDirs, function (resourceDir) {
                _this.$fs.deleteDirectory(path.join(_this.getAppResourcesDestinationDirectoryPath().wait(), resourceDir)).wait();
            });
        }).future()();
    };
    AndroidProjectService.prototype.preparePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            var pluginPlatformsFolderPath = _this.getPluginPlatformsFolderPath(pluginData, AndroidProjectService.ANDROID_PLATFORM_NAME);
            var libsFolderPath = path.join(pluginPlatformsFolderPath, AndroidProjectService.LIBS_FOLDER_NAME);
            if (_this.$fs.exists(libsFolderPath).wait()) {
                _this.addLibrary(libsFolderPath).wait();
            }
        }).future()();
    };
    AndroidProjectService.prototype.removePluginNativeCode = function (pluginData) {
        var _this = this;
        return (function () {
            try {
                var pluginPlatformsFolderPath = _this.getPluginPlatformsFolderPath(pluginData, AndroidProjectService.ANDROID_PLATFORM_NAME);
                var libsFolderPath = path.join(pluginPlatformsFolderPath, AndroidProjectService.LIBS_FOLDER_NAME);
                if (_this.$fs.exists(libsFolderPath).wait()) {
                    var pluginJars = _this.$fs.enumerateFilesInDirectorySync(libsFolderPath);
                    _.each(pluginJars, function (jarName) { return _this.$fs.deleteFile(path.join(libsFolderPath, jarName)).wait(); });
                }
            }
            catch (e) {
                if (e.code === "ENOENT") {
                    _this.$logger.debug("No native code jars found: " + e.message);
                }
                else {
                    throw e;
                }
            }
        }).future()();
    };
    AndroidProjectService.prototype.afterPrepareAllPlugins = function () {
        return Future.fromResult();
    };
    AndroidProjectService.prototype.deploy = function (deviceIdentifier) {
        var _this = this;
        return (function () {
            var adb = _this.$injector.resolve(androidDebugBridgePath.AndroidDebugBridge, { identifier: deviceIdentifier });
            var deviceRootPath = "/data/local/tmp/" + _this.$projectData.projectId;
            adb.executeShellCommand(["rm", "-rf", _this.$mobileHelper.buildDevicePath(deviceRootPath, "fullsync"),
                _this.$mobileHelper.buildDevicePath(deviceRootPath, "sync"),
                _this.$mobileHelper.buildDevicePath(deviceRootPath, "removedsync")]).wait();
        }).future()();
    };
    AndroidProjectService.prototype.canUseGradle = function (frameworkVersion) {
        var _this = this;
        return (function () {
            if (!_this._canUseGradle) {
                if (!frameworkVersion) {
                    _this.$projectDataService.initialize(_this.$projectData.projectDir);
                    frameworkVersion = _this.$projectDataService.getValue(_this.platformData.frameworkPackageName).wait().version;
                }
                _this._canUseGradle = semver.gte(frameworkVersion, AndroidProjectService.MIN_RUNTIME_VERSION_WITH_GRADLE);
            }
            return _this._canUseGradle;
        }).future()();
    };
    AndroidProjectService.prototype.getLibraryPath = function (libraryName) {
        return path.join(this.$projectData.projectDir, "lib", this.platformData.normalizedPlatformName, libraryName);
    };
    AndroidProjectService.prototype.copy = function (projectRoot, frameworkDir, files, cpArg) {
        var paths = files.split(' ').map(function (p) { return path.join(frameworkDir, p); });
        shell.cp(cpArg, paths, projectRoot);
    };
    AndroidProjectService.prototype.spawn = function (command, args, opts) {
        if (this.$hostInfo.isWindows) {
            args.unshift('/s', '/c', command);
            command = process.env.COMSPEC || 'cmd.exe';
        }
        return this.$childProcess.spawnFromEvent(command, args, "close", opts || { stdio: "inherit" });
    };
    AndroidProjectService.prototype.getAntArgs = function (configuration, projectRoot) {
        var args = [configuration, "-f", path.join(projectRoot, "build.xml")];
        if (configuration === "release") {
            if (this.$options.keyStorePath) {
                args = args.concat(["-Dkey.store", this.$options.keyStorePath]);
            }
            if (this.$options.keyStorePassword) {
                args = args.concat(["-Dkey.store.password", this.$options.keyStorePassword]);
            }
            if (this.$options.keyStoreAlias) {
                args = args.concat(["-Dkey.alias", this.$options.keyStoreAlias]);
            }
            if (this.$options.keyStoreAliasPassword) {
                args = args.concat(["-Dkey.alias.password", this.$options.keyStoreAliasPassword]);
            }
        }
        args = args.concat(["-Dns.resources", path.join(__dirname, "../../resources/tools")]);
        return args;
    };
    AndroidProjectService.prototype.validatePackageName = function (packageName) {
        if (!/^[a-zA-Z]+(\.[a-zA-Z0-9][a-zA-Z0-9_]*)+$/.test(packageName)) {
            this.$errors.fail("Package name must look like: com.company.Name");
        }
        if (/\b[Cc]lass\b/.test(packageName)) {
            this.$errors.fail("class is a reserved word");
        }
    };
    AndroidProjectService.prototype.validateProjectName = function (projectName) {
        if (projectName === '') {
            this.$errors.fail("Project name cannot be empty");
        }
        if (/^[0-9]/.test(projectName)) {
            this.$errors.fail("Project name must not begin with a number");
        }
    };
    AndroidProjectService.prototype.getTargetFromAndroidManifest = function () {
        var _this = this;
        return (function () {
            var versionInManifest;
            if (_this.$fs.exists(_this.platformData.configurationFilePath).wait()) {
                var targetFromAndroidManifest = _this.$fs.readText(_this.platformData.configurationFilePath).wait();
                if (targetFromAndroidManifest) {
                    var match = targetFromAndroidManifest.match(/.*?android:targetSdkVersion=\"(.*?)\"/);
                    if (match && match[1]) {
                        versionInManifest = match[1];
                    }
                }
            }
            return versionInManifest;
        }).future()();
    };
    AndroidProjectService.prototype.checkJava = function () {
        var _this = this;
        return (function () {
            try {
                var javaVersion = _this.$sysInfo.getSysInfo().javaVer;
                if (semver.lt(javaVersion, AndroidProjectService.MIN_JAVA_VERSION)) {
                    _this.$errors.failWithoutHelp("Your current java version is " + javaVersion + ". NativeScript CLI needs at least " + AndroidProjectService.MIN_JAVA_VERSION + " version to work correctly. Ensure that you have at least " + AndroidProjectService.MIN_JAVA_VERSION + " java version installed and try again.");
                }
            }
            catch (error) {
                _this.$errors.failWithoutHelp("Error executing command 'java'. Make sure you have java installed and added to your PATH.");
            }
        }).future()();
    };
    AndroidProjectService.prototype.checkAnt = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec("ant -version").wait();
            }
            catch (error) {
                _this.$errors.fail("Error executing commands 'ant', make sure you have ant installed and added to your PATH.");
            }
        }).future()();
    };
    AndroidProjectService.prototype.checkGradle = function () {
        var _this = this;
        return (function () {
            if (!_this.$sysInfo.getSysInfo().gradleVer) {
                _this.$errors.failWithoutHelp("Error executing commands 'gradle'. Make sure you have Gradle installed and its bin directory added to your PATH.");
            }
            _this.$androidToolsInfo.validateInfo({ showWarningsAsErrors: true, validateTargetSdk: true }).wait();
        }).future()();
    };
    AndroidProjectService.prototype.checkAndroid = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec('android list targets').wait();
            }
            catch (error) {
                if (error.message.match(/command\snot\sfound/)) {
                    _this.$errors.fail("The command \"android\" failed. Make sure you have the latest Android SDK installed, and the \"android\" command (inside the tools/ folder) is added to your path.");
                }
                else {
                    _this.$errors.fail("An error occurred while listing Android targets");
                }
            }
        }).future()();
    };
    AndroidProjectService.prototype.symlinkDirectory = function (directoryName, projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            _this.$fs.createDirectory(path.join(projectRoot, directoryName)).wait();
            var directoryContent = _this.$fs.readDirectory(path.join(frameworkDir, directoryName)).wait();
            _.each(directoryContent, function (file) {
                var sourceFilePath = path.join(frameworkDir, directoryName, file);
                var destinationFilePath = path.join(projectRoot, directoryName, file);
                if (_this.$fs.getFsStats(sourceFilePath).wait().isFile()) {
                    _this.$fs.symlink(sourceFilePath, destinationFilePath).wait();
                }
                else {
                    _this.$fs.symlink(sourceFilePath, destinationFilePath, "dir").wait();
                }
            });
        }).future()();
    };
    AndroidProjectService.VALUES_DIRNAME = "values";
    AndroidProjectService.VALUES_VERSION_DIRNAME_PREFIX = AndroidProjectService.VALUES_DIRNAME + "-v";
    AndroidProjectService.ANDROID_PLATFORM_NAME = "android";
    AndroidProjectService.LIBS_FOLDER_NAME = "libs";
    AndroidProjectService.MIN_JAVA_VERSION = "1.7.0";
    AndroidProjectService.MIN_RUNTIME_VERSION_WITH_GRADLE = "1.3.0";
    return AndroidProjectService;
})(projectServiceBaseLib.PlatformProjectServiceBase);
$injector.register("androidProjectService", AndroidProjectService);
