///<reference path="../.d.ts"/>
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var androidLiveSyncServiceLib = require("../common/mobile/android/android-livesync-service");
var constants = require("../constants");
var usbLivesyncServiceBaseLib = require("../common/services/usb-livesync-service-base");
var path = require("path");
var semver = require("semver");
var Future = require("fibers/future");
var UsbLiveSyncService = (function (_super) {
    __extends(UsbLiveSyncService, _super);
    function UsbLiveSyncService($devicesServices, $fs, $mobileHelper, $localToDevicePathDataFactory, $options, $platformsData, $projectData, $deviceAppDataFactory, $logger, $injector, $platformService, $dispatcher, $childProcess, $iOSEmulatorServices, $devicePlatformsConstants, $projectDataService, $prompter, $hostInfo) {
        _super.call(this, $devicesServices, $mobileHelper, $localToDevicePathDataFactory, $logger, $options, $deviceAppDataFactory, $fs, $dispatcher, $injector, $childProcess, $iOSEmulatorServices, $hostInfo);
        this.$platformsData = $platformsData;
        this.$projectData = $projectData;
        this.$platformService = $platformService;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$projectDataService = $projectDataService;
        this.$prompter = $prompter;
        this.excludedProjectDirsAndFiles = [
            "app_resources"
        ];
    }
    UsbLiveSyncService.prototype.liveSync = function (platform) {
        var _this = this;
        return (function () {
            platform = platform || _this.initialize(platform).wait();
            var platformLowerCase = platform ? platform.toLowerCase() : null;
            var platformData = _this.$platformsData.getPlatformData(platformLowerCase);
            if (platformLowerCase === _this.$devicePlatformsConstants.Android.toLowerCase()) {
                _this.$projectDataService.initialize(_this.$projectData.projectDir);
                var frameworkVersion = _this.$projectDataService.getValue(platformData.frameworkPackageName).wait().version;
                if (semver.lt(frameworkVersion, "1.2.1")) {
                    var shouldUpdate = _this.$prompter.confirm("You need Android Runtime 1.2.1 or later for LiveSync to work properly. Do you want to update your runtime now?").wait();
                    if (shouldUpdate) {
                        _this.$platformService.updatePlatforms([_this.$devicePlatformsConstants.Android.toLowerCase()]).wait();
                    }
                    else {
                        return;
                    }
                }
            }
            _this.$platformService.preparePlatform(platform).wait();
            var projectFilesPath = path.join(platformData.appDestinationDirectoryPath, constants.APP_FOLDER_NAME);
            var notInstalledAppOnDeviceAction = function (device) {
                return _this.$platformService.deployOnDevice(platform);
            };
            var notRunningiOSSimulatorAction = function () {
                return _this.$platformService.deployOnEmulator(_this.$devicePlatformsConstants.iOS.toLowerCase());
            };
            var beforeLiveSyncAction = function (device, deviceAppData) {
                var platformSpecificUsbLiveSyncService = _this.resolveUsbLiveSyncService(platform || _this.$devicesServices.platform, device);
                if (platformSpecificUsbLiveSyncService.beforeLiveSyncAction) {
                    return platformSpecificUsbLiveSyncService.beforeLiveSyncAction(deviceAppData);
                }
                return Future.fromResult();
            };
            var beforeBatchLiveSyncAction = function (filePath) {
                return (function () {
                    var projectFileInfo = _this.getProjectFileInfo(filePath);
                    var result = path.join(projectFilesPath, path.relative(path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME), projectFileInfo.onDeviceName));
                    if (filePath.indexOf(path.join(constants.APP_FOLDER_NAME, constants.APP_RESOURCES_FOLDER_NAME, platformData.normalizedPlatformName)) > -1) {
                        var appResourcesRelativePath = path.relative(path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME, constants.APP_RESOURCES_FOLDER_NAME, platformData.normalizedPlatformName), filePath);
                        result = path.join(platformData.platformProjectService.getAppResourcesDestinationDirectoryPath().wait(), appResourcesRelativePath);
                    }
                    if (filePath.indexOf(path.join(constants.APP_FOLDER_NAME, constants.APP_RESOURCES_FOLDER_NAME)) > -1 &&
                        filePath.indexOf(path.join(constants.APP_FOLDER_NAME, constants.APP_RESOURCES_FOLDER_NAME, platformData.normalizedPlatformName)) === -1) {
                        _this.$logger.warn("Unable to sync " + filePath + ".");
                        return null;
                    }
                    return result;
                }).future()();
            };
            var iOSSimulatorRelativeToProjectBasePathAction = function (projectFile) {
                return path.join(constants.APP_FOLDER_NAME, path.dirname(projectFile.split("/" + constants.APP_FOLDER_NAME + "/")[1]));
            };
            var watchGlob = path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME);
            var platformSpecificLiveSyncServices = {
                android: AndroidUsbLiveSyncService,
                ios: IOSUsbLiveSyncService
            };
            var localProjectRootPath = platform.toLowerCase() === "ios" ? platformData.appDestinationDirectoryPath : null;
            _this.sync(platform, _this.$projectData.projectId, projectFilesPath, _this.excludedProjectDirsAndFiles, watchGlob, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, notRunningiOSSimulatorAction, localProjectRootPath, beforeLiveSyncAction, beforeBatchLiveSyncAction, iOSSimulatorRelativeToProjectBasePathAction).wait();
        }).future()();
    };
    UsbLiveSyncService.prototype.preparePlatformForSync = function (platform) {
        this.$platformService.preparePlatform(platform).wait();
    };
    UsbLiveSyncService.prototype.resolveUsbLiveSyncService = function (platform, device) {
        var platformSpecificUsbLiveSyncService = null;
        if (platform.toLowerCase() === "android") {
            platformSpecificUsbLiveSyncService = this.$injector.resolve(AndroidUsbLiveSyncService, { _device: device });
        }
        else if (platform.toLowerCase() === "ios") {
            platformSpecificUsbLiveSyncService = this.$injector.resolve(IOSUsbLiveSyncService, { _device: device });
        }
        return platformSpecificUsbLiveSyncService;
    };
    return UsbLiveSyncService;
})(usbLivesyncServiceBaseLib.UsbLiveSyncServiceBase);
exports.UsbLiveSyncService = UsbLiveSyncService;
$injector.register("usbLiveSyncService", UsbLiveSyncService);
var IOSUsbLiveSyncService = (function () {
    function IOSUsbLiveSyncService(_device) {
        this._device = _device;
    }
    Object.defineProperty(IOSUsbLiveSyncService.prototype, "device", {
        get: function () {
            return this._device;
        },
        enumerable: true,
        configurable: true
    });
    IOSUsbLiveSyncService.prototype.restartApplication = function (deviceAppData) {
        return this.device.applicationManager.restartApplication(deviceAppData.appIdentifier);
    };
    return IOSUsbLiveSyncService;
})();
exports.IOSUsbLiveSyncService = IOSUsbLiveSyncService;
var AndroidUsbLiveSyncService = (function (_super) {
    __extends(AndroidUsbLiveSyncService, _super);
    function AndroidUsbLiveSyncService(_device, $fs, $mobileHelper, $options) {
        _super.call(this, _device, $fs, $mobileHelper);
        this.$options = $options;
    }
    AndroidUsbLiveSyncService.prototype.restartApplication = function (deviceAppData, localToDevicePaths) {
        var _this = this;
        return (function () {
            _this.device.adb.executeShellCommand(["chmod", "777", deviceAppData.deviceProjectRootPath]).wait();
            _this.device.adb.executeShellCommand(["chmod", "777", ("/data/local/tmp/" + deviceAppData.appIdentifier)]).wait();
            if (_this.$options.companion) {
                var commands = [_this.liveSyncCommands.SyncFilesCommand()];
                _this.livesync(deviceAppData.appIdentifier, deviceAppData.deviceProjectRootPath, commands).wait();
            }
            else {
                var devicePathRoot = "/data/data/" + deviceAppData.appIdentifier + "/files";
                var devicePath = _this.$mobileHelper.buildDevicePath(devicePathRoot, "code_cache", "secondary_dexes", "proxyThumb");
                _this.device.adb.executeShellCommand(["rm", "-rf", devicePath]).wait();
            }
            _this.device.applicationManager.restartApplication(deviceAppData.appIdentifier).wait();
        }).future()();
    };
    AndroidUsbLiveSyncService.prototype.beforeLiveSyncAction = function (deviceAppData) {
        var _this = this;
        return (function () {
            var deviceRootPath = "/data/local/tmp/" + deviceAppData.appIdentifier;
            _this.device.adb.executeShellCommand(["rm", "-rf", _this.$mobileHelper.buildDevicePath(deviceRootPath, "fullsync")]).wait();
            _this.device.adb.executeShellCommand(["rm", "-rf", _this.$mobileHelper.buildDevicePath(deviceRootPath, "sync")]).wait();
            _this.device.adb.executeShellCommand(["rm", "-rf", _this.$mobileHelper.buildDevicePath(deviceRootPath, "removedsync")]).wait();
        }).future()();
    };
    return AndroidUsbLiveSyncService;
})(androidLiveSyncServiceLib.AndroidLiveSyncService);
exports.AndroidUsbLiveSyncService = AndroidUsbLiveSyncService;
